﻿

import java.awt.*;
import java.awt.event.*;  // scrollbar adjust
import java.awt.FontMetrics.*;
import java.io.IOException;

import javax.swing.*;


class HexPanel extends JPanel implements AdjustmentListener {
    Font f1 = new Font( "Courier New", Font.PLAIN, 14 ); // Courier New
    Font fMitte = new Font( "Verdana", Font.PLAIN, 14 ); // Courier New
    
    private Scrollbar _scrollbar;
    private int _fontsize=12;
    private HexFile _hexfile;
    private int _colCount=16;
    private int _offset=0;
    private int _linkerRand=0;
    private int _rechterRand=100;
    private int _rowHeight=10;
    private int _wFF=10;
    private boolean _bFirst=true;
    private int _blockRowCount=25;  // geschaetzt
    private String _sFilename;
    
   
  public HexPanel(String sFilename) {
    _sFilename = sFilename;
    setLayout( new BorderLayout() );
    _scrollbar = new Scrollbar(Scrollbar.VERTICAL);
    add( _scrollbar, BorderLayout.EAST);
    this._hexfile = new HexFile(sFilename);
    
    _scrollbar.setBlockIncrement(_colCount*25);
    _scrollbar.setUnitIncrement(1);                    
    _scrollbar.setMinimum(0);
    _scrollbar.setMaximum(100);  // geschaetzt
    _scrollbar.addAdjustmentListener(this);

    // initDatas();
  }
  
   public void loadFile(String sFilename) {
       _hexfile.loadFile(sFilename);
       long filesize = _hexfile.getFilesize();
       int lastrow = (int) Math.ceil((double)filesize / _colCount)-1;
        _scrollbar.setBlockIncrement(_colCount);
        _scrollbar.setUnitIncrement(1);
        _scrollbar.setMinimum(0);
        _scrollbar.setMaximum(lastrow);
        _scrollbar.setValue(0);
        _offset=0;
        System.out.println("lastrow: "+lastrow);
        initDatas();
   } // loadFile
   

   public void setFontSize(int size) {
     this._fontsize = size;
     f1 = new Font("Courier New", Font.PLAIN, size);  
     fMitte = new Font( "Verdana", Font.PLAIN, size ); // Courier New
     _bFirst = true;       
     initDatas();
     updateUI();
   }

   private void initDatas() {
     Graphics g = this.getGraphics();
     g.setFont(f1);
  
     FontMetrics fontMetrics = g.getFontMetrics(f1);
     _rowHeight = fontMetrics.getHeight();
     _linkerRand = fontMetrics.stringWidth("00000:_");
     _wFF = fontMetrics.stringWidth("FF ");
     _rechterRand = 10+_linkerRand+(_wFF<<4);
   }
     

  
   
 @Override
    public void adjustmentValueChanged(AdjustmentEvent e) {
        // throw new UnsupportedOperationException("Not supported yet.");
        System.out.println("Offset: "+_scrollbar.getValue() );
        _offset= _scrollbar.getValue()*_colCount;
        updateUI();
    }   

 @Override
 public void paint (Graphics g) {
    Graphics2D g2d = (Graphics2D)g;
    g.setFont(f1);
   
    g.setColor(Color.white);
    g.fillRect(0,0, getWidth(), getHeight() ); 
    if (_hexfile.getFilename().equals("")) {
      g.setColor(Color.blue);
      g.drawString( "Keine Datei ausgewählt", 10,_rowHeight<<1);
      return;
    }

    g.setColor( new Color(20,20,255) );
    int y=_rowHeight; // y ist immer unten
    int offset=_offset;
    int ii=0;
    if (_bFirst) _blockRowCount = 0;
    while ( (y)<getHeight() ) {
      byte[] feld = _hexfile.loadRow(offset, _colCount);
      if (feld==null)  break;
      
      String s = Integer.toHexString(offset).toUpperCase();
      while (s.length()<5) {
        s="0"+s;
      }
      g.setColor(Color.black);
      g.setFont(f1);
      g.drawString( s, 10,y);
      g.setColor(Color.blue);
      String sMitte="";
      String sRechts="";
      
      for (int i=0; i<feld.length; i++) {
          int value = (int) feld[i];  // Problem signed byte
          if (value<0) {
              value+=256;
          }
        s = Integer.toHexString( value ).toUpperCase();
        if (s.length()==1) s+="0";
        g.setFont(fMitte);
        g.drawString(s, 10+_linkerRand + _wFF*i,y); // zeichnen mittels TAB !
        sMitte+=s+" ";
        if ( value<32) // Sonderzeichen 
            sRechts+=".";
        else
            sRechts+=(char) value;
      }      
      // g.drawString( sMitte, 10+_linkerRand,y);
      
      g.setFont(f1);
      g.setColor(Color.black);
      g.drawString( sRechts, _rechterRand,y);
      
      y+=_rowHeight;
      offset+=_colCount;   
      ii++;      
      if (_bFirst) _blockRowCount++;
    }
    if (_bFirst)
    {
        _bFirst = false;    
        _scrollbar.setBlockIncrement( _blockRowCount);  // Anzahl der Zeilen pro Fenster
        System.out.println("_blockRowCount:"+_blockRowCount);
    }
  }  // paint

 
 
   public void Export2BIFF(String sFilename) {
      int COLCOUNT = 16;
      int offset=0;       
      
      long laenge = this._hexfile.getFilesize();
      int c = (int) laenge / COLCOUNT;  // maximal 60000 Zeilen
      if (c>60000) {
          // ceil rundet auf, immer Vielfache von 16
          int count = (int) Math.ceil( (double) laenge/60000.0   );  // z. B. 19
          // aufrunden auf Vielfache von 16
          COLCOUNT = (int) Math.ceil( count/16.0   )*16;  // z. B. 19 auf 32
      }     
      
      Export2BIFF excel = new Export2BIFF(sFilename);
      excel.openFile();
      
      excel.Write(0,0,sFilename); // col row Value
      excel.Write(0,1,"Adresse"); // col row Value
      for (int i=0; i<COLCOUNT; i++) {
          excel.Write(1+i, 1, i ); // col row Value
      }
      excel.Write(COLCOUNT+1, 1, "Text" ); // col row Value
      int row=1;
      while ( true ) {
        byte[] feld = _hexfile.loadRow(offset, COLCOUNT);
        if (feld==null)  break;
        row++;

        String sAdr = Integer.toHexString(offset).toUpperCase();
        while (sAdr.length()<5) { // oder mit Format
            sAdr="0"+sAdr;
        }
        excel.Write(0, row,sAdr); // col row Value
        
        String sMitte="";
        String sRechts="";

        for (int i=0; i<feld.length; i++) {
            int value = (int) feld[i];  // Problem signed byte
            if (value<0) {
                value+=256;
            }
            sMitte = Integer.toHexString( value ).toUpperCase();
            if (sMitte.length()==1) sMitte+="0";
            excel.Write(1+i,row,sMitte); // col row Value
            if ( value<32) // Sonderzeichen entfernen
                sRechts+=".";
            else
                sRechts+=(char) value;
        }      
        excel.Write(1+COLCOUNT,row, sRechts ); // col row Value
        offset+=COLCOUNT;   
      }
      
      excel.closeFile();
      StartExcel(sFilename);     
   }

    private void StartExcel(String sFilename) {
        String sExcel ="C:\\Program Files (x86)\\Microsoft Office\\Office14\\EXCEL.EXE";
        try {
        Runtime.getRuntime().exec(sExcel+" "+sFilename);
        }
        catch (IOException e1) {
        System.out.println("Error beim Starten von Excel");
        }
    }
   
}



